require 'thor'
require 'yell'

module OnetableTerminator
  class CLI < Thor
    class_option :debug,
    default: false,
    type: :boolean,
    desc: 'Runs onetable-terminator in debug mode.'
    class_option :"dry-run",
    default: false,
    type: :boolean,
    desc: 'Runs without making changes to current iptables. Automatically in debug mode.'

    option :driver,
    default: 'fw_driver',
    type: :string,
    desc: 'OpenNebula\'s VNM driver'
    desc 'clean', 'Removes OpenNebula\'s forgotten iptables rules. Default command.'
    def clean
      initialize_logger options

      lock = OnetableTerminator::Lock.new options[:driver]
      lock.lock

      core = OnetableTerminator::Core.new options[:"dry-run"]
      core.run

      lock.unlock
    end

    desc 'version', 'Prints onetable-terminator version'
    def version
      $stdout.puts OnetableTerminator::VERSION
    end

    default_task :clean

    private

    def initialize_logger(parameters)
      logging_level = 'INFO'
      logging_level = 'DEBUG' if parameters[:debug]

      Yell.new :stdout, name: Object, level: logging_level.downcase, format: Yell::DefaultFormat
      Object.send :include, Yell::Loggable

      logger.debug 'Running in debug mode...'
    end
  end
end
