module OnetableTerminator
  class Core
    attr_reader :exec, :rules_handler

    def initialize(dry_run = false)
      @exec = OnetableTerminator::Iptables::Exec.new dry_run
      @rules_handler = OnetableTerminator::Iptables::RulesHandler.new
    end

    def run
      rules = retrieve_redundant_rules

      if rules.empty?
        logger.debug 'Found no redundant rules, nothing to delete'
        return
      end

      log_redundant_rules rules
      delete_redundant_rules rules
    end

    private

    def retrieve_redundant_rules
      nics = exec.load_opennebula_chain
      rules = rules_handler.find_redundant_rules(nics)
      rules.sort! { |left, right| left.number <=> right.number }
      rules.reverse!

      rules
    end

    def log_redundant_rules(rules)
      logger.info 'Found these redundant rules:'
      rules.each { |rule| logger.info rule.raw_line.inspect }
    end

    def delete_redundant_rules(rules)
      logger.debug 'Deleting redundant rules and chains...'
      rules.each do |rule|
        exec.delete_opennebula_rule rule.number
        exec.flush_opennebula_chain rule.target
        exec.delete_opennebula_chain rule.target
      end
    end
  end
end
