module OnetableTerminator
  module Iptables
    class Parser
      COLUMN_COUNT = 11
      TARGET_COLUMN_COUNT = 4
      TARGET_VM_ID_COLUMN = 1
      TARGET_IO_COLUMN = 3
      NUMBER_COLUMN = 0
      TARGET_COLUMN = 1
      NIC_COLUMN = 9

      def parse_rules(output)
        nics = {}
        lines = output.lines.to_a
        normalize_output!(lines)

        lines.each do |line|
          parsed_line = parse_line line
          nic_name = parsed_line[:nic]

          nic = nics[nic_name]
          unless nic
            nic = OnetableTerminator::Structures::Nic.new nic_name
            nics[nic_name] = nic
          end
          nic.add_rule OnetableTerminator::Structures::Rule.new parsed_line
        end

        nics.values
      end

      private

      def normalize_output!(lines)
        lines.shift 2

        unless lines.empty?
          accept_line = lines.pop
          raise OnetableTerminator::Errors::ParsingError, "Missing last ACCEPT rule" unless accept_line =~ /ACCEPT/
        end
      end

      def parse_line(line)
        logger.debug "Parsing line #{line.inspect}"
        splitted_line = line.split
        raise OnetableTerminator::Errors::ParsingError, "Cannot parse line #{line.inspect}" unless splitted_line.size == COLUMN_COUNT

        target = parse_target splitted_line[TARGET_COLUMN]
        number = splitted_line[NUMBER_COLUMN]

        raise OnetableTerminator::Errors::ParsingError, "Cannot parse line number for line #{line.inspect}" unless number =~ /\A\d+\z/

        parsed_line = { number: number.to_i, nic: splitted_line[NIC_COLUMN], raw_line: line, target: splitted_line[TARGET_COLUMN] }.merge target
        logger.debug "Line parsed: #{parsed_line.inspect}"

        parsed_line
      end

      def parse_target(target)
        slices = target.split('-', TARGET_COLUMN_COUNT)

        raise OnetableTerminator::Errors::ParsingError, "Cannot parse target #{target.inspect}" unless slices.size == TARGET_COLUMN_COUNT

        vm_id = slices[TARGET_VM_ID_COLUMN]
        io = slices[TARGET_IO_COLUMN]

        raise OnetableTerminator::Errors::ParsingError, "Cannot parse VM ID for chain #{target.inspect}" unless vm_id =~ /\A\d+\z/
        raise OnetableTerminator::Errors::ParsingError, "Cannot detect input/output for chain #{target.inspect}" unless io == 'i' || io == 'o'

        { vm_id: vm_id.to_i, io: io }
      end
    end
  end
end
